@@ -36,6 +36,7 @@
 #include <TargetConditionals.h>
 #if TARGET_OS_IPHONE || TARGET_IPHONE_SIMULATOR
 #include <CFNetwork/CFNetwork.h>
+#include <Security/Security.h>
 #else
 #include <CoreServices/CoreServices.h>
 #endif
@@ -129,6 +130,11 @@ static ssize_t mailstream_low_cfstream_write(mailstream_low * s,
 static void mailstream_low_cfstream_free(mailstream_low * s);
 static int mailstream_low_cfstream_get_fd(mailstream_low * s);
 static void mailstream_low_cfstream_cancel(mailstream_low * s);
+static carray * mailstream_low_cfstream_get_certificate_chain(mailstream_low * s);
+
+static int mailstream_low_cfstream_setup_idle(mailstream_low * s);
+static int mailstream_low_cfstream_unsetup_idle(mailstream_low * s);
+static int mailstream_low_cfstream_interrupt_idle(mailstream_low * s);
 
 static mailstream_low_driver local_mailstream_cfstream_driver = {
   /* mailstream_read */ mailstream_low_cfstream_read,
@@ -137,7 +143,11 @@ static mailstream_low_driver local_mailstream_cfstream_driver = {
   /* mailstream_get_fd */ mailstream_low_cfstream_get_fd,
   /* mailstream_free */ mailstream_low_cfstream_free,
   /* mailstream_cancel */ mailstream_low_cfstream_cancel,
-  /* mailstream_get_cancel_fd */ NULL,
+  /* mailstream_get_cancel */ NULL,
+  /* mailstream_get_certificate_chain */ mailstream_low_cfstream_get_certificate_chain,
+  /* mailstream_setup_idle */ mailstream_low_cfstream_setup_idle,
+  /* mailstream_unsetup_idle */ mailstream_low_cfstream_unsetup_idle,
+  /* mailstream_interrupt_idle */ mailstream_low_cfstream_interrupt_idle,
 };
 
 mailstream_low_driver * mailstream_cfstream_driver =
@@ -169,11 +179,13 @@ static void cfstream_data_free(struct mailstream_cfstream_data * cfstream_data)
 static void cfstream_data_close(struct mailstream_cfstream_data * cfstream_data)
 {
   if (cfstream_data->writeStream != NULL) {
+    CFWriteStreamSetClient(cfstream_data->writeStream, kCFStreamEventNone, NULL, NULL);
     CFWriteStreamClose(cfstream_data->writeStream);
     CFRelease(cfstream_data->writeStream);
     cfstream_data->writeStream = NULL;
   }
   if (cfstream_data->readStream != NULL) {
+    CFReadStreamSetClient(cfstream_data->readStream, kCFStreamEventNone, NULL, NULL);
     CFReadStreamClose(cfstream_data->readStream);
     CFRelease(cfstream_data->readStream);
     cfstream_data->readStream = NULL;
@@ -701,6 +713,17 @@ static int wait_runloop(mailstream_low * s, int wait_state)
     int r;
     int done;
     
+    if (cfstream_data->cancelled) {
+      error = WAIT_RUNLOOP_EXIT_CANCELLED;
+      break;
+    }
+    if (cfstream_data->state == STATE_WAIT_IDLE) {
+      if (cfstream_data->idleInterrupted) {
+        error = WAIT_RUNLOOP_EXIT_INTERRUPTED;
+        break;
+      }
+    }
+
     done = 0;
     switch (cfstream_data->state) {
       case STATE_OPEN_READ_DONE:
@@ -764,16 +787,6 @@ static int wait_runloop(mailstream_low * s, int wait_state)
       error = WAIT_RUNLOOP_EXIT_TIMEOUT;
       break;
     }
-    if (cfstream_data->cancelled) {
-      error = WAIT_RUNLOOP_EXIT_CANCELLED;
-      break;
-    }
-    if (cfstream_data->state == STATE_WAIT_IDLE) {
-      if (cfstream_data->idleInterrupted) {
-        error = WAIT_RUNLOOP_EXIT_INTERRUPTED;
-        break;
-      }
-    }
   }
   
   if (read_scheduled) {
@@ -957,16 +970,36 @@ int mailstream_cfstream_set_ssl_enabled(mailstream * s, int ssl_enabled)
     //fprintf(stderr, "is not ssl\n");
   }
   
-  r = wait_runloop(s->low, STATE_WAIT_SSL);
-  if (r != WAIT_RUNLOOP_EXIT_NO_ERROR) {
-    return -1;
+  // We need to investigate more about how to establish a STARTTLS connection.
+  // For now, wait until we get the certificate chain.
+  while (1) {
+    r = wait_runloop(s->low, STATE_WAIT_SSL);
+    if (r != WAIT_RUNLOOP_EXIT_NO_ERROR) {
+      return -1;
+    }
+    if (cfstream_data->writeSSLResult < 0)
+      return -1;
+    if (cfstream_data->readSSLResult < 0)
+      return -1;
+    
+    SecTrustRef secTrust = (SecTrustRef)CFReadStreamCopyProperty(cfstream_data->readStream, kCFStreamPropertySSLPeerTrust);
+    if (secTrust == NULL) {
+      // No trust, wait more.
+      continue;
+    }
+    
+    // SecTrustEvaluate() needs to be called before SecTrustGetCertificateCount() in Mac OS X <= 10.8
+    SecTrustEvaluate(secTrust, NULL);
+    CFIndex count = SecTrustGetCertificateCount(secTrust);
+    CFRelease(secTrust);
+    
+    if (count == 0) {
+      // No certificates, wait more.
+      continue;
+    }
+    break;
   }
   
-  if (cfstream_data->writeSSLResult < 0)
-    return -1;
-  if (cfstream_data->readSSLResult < 0)
-    return -1;
-  
   return 0;
 #else
   return -1;
@@ -1028,15 +1061,20 @@ void mailstream_cfstream_set_ssl_level(mailstream * s, int ssl_level)
 }
 
 int mailstream_cfstream_wait_idle(mailstream * s, int max_idle_delay)
+{
+  return mailstream_low_cfstream_wait_idle(s->low, max_idle_delay);
+}
+
+int mailstream_low_cfstream_wait_idle(mailstream_low * low, int max_idle_delay)
 {
 #if HAVE_CFNETWORK
   struct mailstream_cfstream_data * cfstream_data;
   int r;
   
-  cfstream_data = (struct mailstream_cfstream_data *) s->low->data;
+  cfstream_data = (struct mailstream_cfstream_data *) low->data;
   cfstream_data->idleMaxDelay = max_idle_delay;
   
-  r = wait_runloop(s->low, STATE_WAIT_IDLE);
+  r = wait_runloop(low, STATE_WAIT_IDLE);
   switch (r) {
     case WAIT_RUNLOOP_EXIT_TIMEOUT:
       return MAILSTREAM_IDLE_TIMEOUT;
@@ -1055,48 +1093,53 @@ int mailstream_cfstream_wait_idle(mailstream * s, int max_idle_delay)
 static void idleInterruptedPerform(void *info)
 {
   struct mailstream_cfstream_data * cfstream_data;
-  mailstream * s;
+  mailstream_low * s;
   
   s = info;
-  //fprintf(stderr, "interrupt idle\n");
   
-  cfstream_data = (struct mailstream_cfstream_data *) s->low->data;
+  cfstream_data = (struct mailstream_cfstream_data *) s->data;
   cfstream_data->idleInterrupted = true;
 }
 #endif
 
-void mailstream_cfstream_setup_idle(mailstream * s)
+static int mailstream_low_cfstream_setup_idle(mailstream_low * s)
 {
 #if HAVE_CFNETWORK
   struct mailstream_cfstream_data * cfstream_data;
   
-  cfstream_data = (struct mailstream_cfstream_data *) s->low->data;
+  cfstream_data = (struct mailstream_cfstream_data *) s->data;
   cfstream_data->idleInterrupted = false;
   cfstream_data->idleInterruptedContext.info = s;
   cfstream_data->idleInterruptedContext.perform = idleInterruptedPerform;
   cfstream_data->idleInterruptedSource = CFRunLoopSourceCreate(NULL, 0, &cfstream_data->idleInterruptedContext);
+  return 0;
+#else
+  return -1;
 #endif
 }
 
-void mailstream_cfstream_unsetup_idle(mailstream * s)
+static int mailstream_low_cfstream_unsetup_idle(mailstream_low * s)
 {
 #if HAVE_CFNETWORK
   struct mailstream_cfstream_data * cfstream_data;
   
-  cfstream_data = (struct mailstream_cfstream_data *) s->low->data;
+  cfstream_data = (struct mailstream_cfstream_data *) s->data;
   if (cfstream_data->idleInterruptedSource != NULL) {
     CFRelease(cfstream_data->idleInterruptedSource);
     cfstream_data->idleInterruptedSource = NULL;
   }
+  return 0;
+#else
+  return -1;
 #endif
 }
 
-void mailstream_cfstream_interrupt_idle(mailstream * s)
+static int mailstream_low_cfstream_interrupt_idle(mailstream_low * s)
 {
 #if HAVE_CFNETWORK
   struct mailstream_cfstream_data * cfstream_data;
   
-  cfstream_data = (struct mailstream_cfstream_data *) s->low->data;
+  cfstream_data = (struct mailstream_cfstream_data *) s->data;
   
   pthread_mutex_lock(&cfstream_data->runloop_lock);
   
@@ -1108,5 +1151,44 @@ void mailstream_cfstream_interrupt_idle(mailstream * s)
   }
   
   pthread_mutex_unlock(&cfstream_data->runloop_lock);
+  return 0;
+#else
+  return -1;
+#endif
+}
+
+static carray * mailstream_low_cfstream_get_certificate_chain(mailstream_low * s)
+{
+#if HAVE_CFNETWORK
+  struct mailstream_cfstream_data * cfstream_data;
+  unsigned int i;
+  carray * result;
+  
+  cfstream_data = (struct mailstream_cfstream_data *) s->data;
+  SecTrustRef secTrust = (SecTrustRef)CFReadStreamCopyProperty(cfstream_data->readStream, kCFStreamPropertySSLPeerTrust);
+  if (secTrust == NULL)
+    return NULL;
+  
+  // SecTrustEvaluate() needs to be called before SecTrustGetCertificateCount() in Mac OS X <= 10.8
+  SecTrustEvaluate(secTrust, NULL);
+  CFIndex count = SecTrustGetCertificateCount(secTrust);
+  
+  result = carray_new(4);
+  for(i = 0 ; i < count ; i ++) {
+    SecCertificateRef cert = (SecCertificateRef) SecTrustGetCertificateAtIndex(secTrust, i);
+    CFDataRef data = SecCertificateCopyData(cert);
+    CFIndex length = CFDataGetLength(data);
+    const UInt8 * bytes = CFDataGetBytePtr(data);
+    MMAPString * str = mmap_string_sized_new(length);
+    mmap_string_append_len(str, (char*) bytes, length);
+    carray_add(result, str, NULL);
+    CFRelease(data);
+  }
+  
+  CFRelease(secTrust);
+  
+  return result;
+#else
+  return NULL;
 #endif
 }
