@@ -319,6 +319,13 @@ struct mailimap_fetch_att *
 mailimap_fetch_att_new_body_peek_section_partial(struct mailimap_section * section,
 						 uint32_t offset, uint32_t size);
 
+/*
+ creates a mailimap_fetch_att extension
+*/
+    
+struct mailimap_fetch_att *
+    mailimap_fetch_att_new_extension(char * ext_keyword);
+
 /*
   this function creates a mailimap_fetch_type structure to request
   (FLAGS INTERNALDATE RFC822.SIZE ENVELOPE) of a message
@@ -563,8 +570,7 @@ mailimap_search_key_new_unkeyword(char * sk_unkeyword);
 
 struct mailimap_search_key *
 mailimap_search_key_new_header(char * sk_header_name, char * sk_header_value);
-
-
+  
 /*
   this function creates a condition structure to match messages with size
 */
@@ -664,6 +670,39 @@ mailimap_search_key_multiple_add(struct mailimap_search_key * keys,
 				 struct mailimap_search_key * key_item);
 
 
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_arrival(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_cc(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_date(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_from(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_size(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_subject(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_to(int is_reverse);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_multiple(clist * keys);
+
+struct mailimap_sort_key *
+mailimap_sort_key_new_multiple_empty(void);
+
+int
+mailimap_sort_key_multiple_add(struct mailimap_sort_key * keys,
+                               struct mailimap_sort_key * key_item);
+
+
 /*
   this function creates an empty list of flags
 */
